/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *		Lars Kiilerich   <lars[at]v1p[dot]dk>
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview

Injects the eDOM Javascript files into the document.
The files should only be injected once.
After injection the javascript files are immediateluy unloaded.
*/

/**
mozileIsLoaded

Test if files have been loaded already.
Attempting to load javacript files a second time produces errors, even if they have been "unloaded".

Input should be a window object of some sort.

Return values
true : has been injected
false : has not been injected
null : no window, or error.
*/
function mozileIsLoaded( target ) {
	if ( target && ( typeof(target) == "object" ) ) {
		if ( "mozileChromeLoaded" in target ) {
			return true;
		} else {
			return false;
		}
	}
	else {
		return null;	
	}
}

// Set the value of mozileChromeLoaded
//
function mozileSetLoaded( target, value ) {
	if ( typeof(target) == "object" ) {
		if ( "mozileChromeLoaded" in target ) {
			target.mozileChromeLoaded = value;
			return true;
		} else {
			return false;
		}
	}
	else {
		return false;	
	}
}


function mozileInject(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileInject"];
	mozile.bug.write(f,2,"Injection starting");
	
	try {
		if ( mozileIsLoaded(mozileInjectTarget) !== false ) {
			throw "Files already loaded?";
		}

  	var injectPosn;
		if ( mozileInjectTarget.document.documentElement.nodeName.toLowerCase() == "html" ) {
			if ( mozileInjectTarget.document.getElementsByTagName("head").length > 0 ) {
				injectPosn = mozileInjectTarget.document.getElementsByTagName("head")[0];
				mozile.bug.write(f,3,"Injecting into X/HTML");
			}
			//else if no head???
		} else {
			injectPosn = mozileInjectTarget.document.documentElement;
			mozile.bug.write(f,3,"Injecting into XML");
		}
		mozileLoadConfig(mozileInjectTarget, injectPosn);

		var mes = mozileMES.getCurrentMES( mozileInjectTarget );
		mozileMES.loadMES(mozileInjectTarget, mes );
		mozileInjectFiles( mozileInjectTarget, mozile.jsFiles, MOZILEC.EXTENSION_DIR );
		mozileInjectTarget.mozileUndo = new MozileUndo( mozile.undoMax, mozile.keyMax );
		mozileInjectTarget.mozileTags = new MozileTags( false, "mozile-show-tags", MOZILEC.TAGS_XHTML);
		mozileInjectTarget.addEventListener("beforeunload", mozileOnBeforeUnload, false);
		mozileInjectTarget.addEventListener("unload", mozileOnUnload, false);
		mozileAddKeypressListener(mozileInjectTarget);
		// Create global flag to let you know Mozile has been loaded.
		mozileInjectTarget.mozileChromeLoaded = "true";

		//Now unload files...
		mozileUnloadEDOM(mozileInjectTarget);

		mozile.bug.write(f,2,"Injection successful");
	} catch (e) {
		mozile.bug.write(f,1,"Injection error: "+e);
	}
}

/**
Adds the eDOM javascript to the document.

@param {Window} target
@param {Array} files
@param {String} dir
*/
function mozileInjectFiles( target, files, dir ) {
	var f = ["mozileConfiguration.js","mozileInjectFiles"];
	var doc, docEl, head, tag, i;
	try {
		doc = target.document;
		docEl = doc.documentElement;
		if ( (docEl.nodeName == "HTML") && (doc.getElementsByTagName("head").length>0) ) {
		// text/html with a <head>
			head = doc.getElementsByTagName("head")[0];
			for (i=0; i < files.length; i++) {
				tag = scriptTag(null, files[i], dir+files[i]);
				head.appendChild(tag);
			}
		}else if ( (docEl.nodeName == "html") && (doc.getElementsByTagName("head").length>0) ) {
		// xhtml document with a <head>
			head = doc.getElementsByTagName("head")[0];
			for (i=0; i < files.length; i++) {
				tag = scriptTag(MOZILEC.NAMESPACE_XHTML, files[i], dir+files[i]);
				head.appendChild(tag);
			}
		} else {
		// XML document, or funny x/html document without a <head>
		// Should create a head tag as well???
			var firstChild = docEl.firstChild;
			for (i=0; i < files.length; i++) {
				tag = scriptTag(MOZILEC.NAMESPACE_XHTML, files[i], dir+files[i]);
				docEl.insertBefore( tag, firstChild );
			}
		}
		mozile.bug.write(f,3,"eDOM injected");
	} catch (e) {
		mozile.bug.write(f,1,"Error injecting files: "+e);
	}
	
	function scriptTag( namespace, id, src ) {
		var tag;
		if (!namespace) {
			tag = doc.createElement("script");
		} else {
			tag = doc.createElementNS(namespace,"script");
		}
		tag.setAttribute("id", id);
		tag.setAttribute("src", src);
		tag.setAttribute("type","text/javascript");
		return tag;
	}
}

function mozileUnload(target) {
	target.mozileTags.unload(target);
	mozileUnloadEDOM(target);
}


/**
Removes the eDOM javascript.
*/
function mozileUnloadEDOM(target) {
	var f=["mozileConfiguration.js","mozileUnloadEDOM"];
	mozile.bug.write(f,3,"Unloading eDOM");

	for(var i=mozile.jsFiles.length-1; i>=0; i--) {
		var id = mozile.jsFiles[i];
		var node = target.document.getElementById(id);
		if(node) {
			node.parentNode.removeChild(node);
		}
	}
	mozileSetLoaded( target, "false" );
}


function mozileOnUnload(event) {
	var f=["mozileConfiguration.js","mozileOnUnload"];
	try {
		if (mozilePrefs.getBoolPref("mozile.onBool")) {
			//If the document just unloaded is the current mozileTarget perform a Mozile reset.
			var curTarget = mozileGetCurrentTarget();
			if ( curTarget && (event.currentTarget == curTarget) ) {
				mozileClearCurrentTarget();
				mozileClearDisplay();
				mozile.bug.write(f,1,"Mozile clear down: "+curTarget);
			}
		}
  }	catch (e) {
		mozile.bug.write(f,2,"Exception during unload event handler: "+e);
	}
}

/**
Handle before unload events.
<p>This warns the user if there are unsaved changes in the document and gives them
the chance to stay on the page or unload the page (usually navigate away).
The feature needs to be enabled by the "promptOnUnload" user preference setting.

<p>Pop up an alert saying:
<pre>
	Are you sure you want to navigate away from this page?
	...custom message here...
	Press OK to continue, or Cancel to stay on the current page. 
</pre>

<p>Note: The DOM event listener route appears to be buggy.
*/
function mozileOnBeforeUnload(event) {
	var f=["mozileConfiguration.js","mozileOnBeforeUnload"];
	try {
		if(mozilePrefs.getBoolPref("mozile.onBool") && mozilePrefs.getBoolPref("mozile.promptOnUnloadBool")) {
			var mTarg=event.currentTarget;
			if ( mozileIsLoaded(mTarg) ) { //If Mozile injected
				mozile.bug.write(f,1,"Unload event: "+ mTarg.document.documentURI);
  			if(mTarg.document.MOZILE_CURRENT_CONFIG.saved !== "true") {
					//If something to save pop up the special alert;
    			mozile.bug.write(f,1,"Unsaved changes, unload document?");
					var warning = "This page contains unsaved changes.";
    			event.returnValue = warning;
    			event.stopPropagation();
    			event.preventDefault();
				}
			}
		}
  }	catch (e) {
		mozile.bug.write(f,2,"Exception: Nothing to unload? "+e);
	}
}

function mozileScourCSS(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileScourCSS"];
	mozile.bug.write(f,3,"Scouring CSS");
	mozile.bug.write(f,3,"Number of style sheets: "+mozileInjectTarget.document.styleSheets.length);
	var URL = "";
	for(var	i=0; i<mozileInjectTarget.document.styleSheets.length;i++)	{
		try{
			mozile.bug.write(f,3,"Style sheet "+ i +" has "+mozileInjectTarget.document.styleSheets[i].cssRules.length+" rules");
			for	(var j=0; j<mozileInjectTarget.document.styleSheets[i].cssRules.length;j++) {
				var	sel	= mozileInjectTarget.document.styleSheets[i].cssRules[j];
				if(sel.selectorText=="mozileconfiguration")	{
					URL = sel.style.backgroundImage;
					URL = URL.substring(4,URL.length-1);
					mozile.bug.write(f,3,"URL found:"+i+":"+j+":"+URL);
					break;
				}
			}
		} catch(e) {
			// styleSheet is inaccessible, don't worry about it
			mozile.bug.write(f,2,"Problem with style sheet number "+i+"? "+e);
		}
	}
	return URL;
}

// returns an absolute URL computed from a (possibly relative) URL and a starting point (relativeTo)
function mozileAbsoluteURL(URL, relativeTo) {
	var f = ["mozileConfiguration.js","mozileAbsoluteURL"];
	mozile.bug.write(f,3,"Start with: "+relativeTo+"  "+URL);
	if(URL.search("://")>0) {
		mozile.bug.write(f,3,"URL contains '://', so assume it's absolute.");
		return URL;
	}
	relativeTo += " "; // make sure it's a string

  // Remove query string part of already absolute part
  var queryPos = relativeTo.lastIndexOf("?");
  if (queryPos > -1) {
    relativeTo = relativeTo.substring(0, queryPos);
	}

  // Check if given URL is relative to host root or current document
  if (URL.charAt(0) == "/") {
    // Make URL relative
    URL = URL.substring(1);

    // Find beginning of hostname
    var hostPos = relativeTo.search("://")+3;
    if (hostPos < 0) {
      hostPos = 0;
		}

    // Find end of hostname
    hostPos = relativeTo.indexOf("/", hostPos);
    relativeTo = relativeTo.substring(0, hostPos) + "/";
  }

	var last = relativeTo.lastIndexOf("/");
  var dir = relativeTo.substring(0,last+1);
  	// just tacking the relative URL onto the directory of the reference URL seems to work fine: i.e. http://overton.ca/james/mozile/mozile1/../mozileTest.html becomes http://overton.ca/james/mozile/mozile1/mozileTest.html
	mozile.bug.write(f,3,"Try: "+ dir + URL);
	URL = dir+URL;
	return URL;
}

function mozileLoadConfig (mozileInjectTarget, headnode) {
	var f = ["mozileConfiguration.js","mozileLoadConfig"];
	mozile.bug.write(f,3,"Load Config starting");
	
	/* Look	for	an RSD declaration and load	configuration if present */

	var	rsdtype	= "application/rsd+xml";
//	var	rsdns =	"http://archipelago.phrasewise.com/rsd";
	var	presetDisallowXPath	= "//setting[@name='savePresetDisallow']";
	var	methodXPath	= "//setting[@name='saveMethod']";
	var	methodDisallowXPath	= "//setting[@name='saveMethodDisallow']";
	var	contentXPath = "//setting[@name='saveContent']";
	var	contentDisallowXPath = "//setting[@name='saveContentDisallow']";
	var httpSavePathXPath = "//setting[@name='httpSavePath']";
	var httpSavePathModifyXPath = "//setting[@name='httpSavePathModify']";
	var saveHTMLCaseXPath = "//setting[@name='saveHTMLCase']";
	var saveHTMLCompatibilityXPath = "//setting[@name='saveHTMLCompatibility']";
	var saveEntitiesXPath = "//setting[@name='saveEntities']";
	var saveCharsetXPath = "//setting[@name='saveCharset']";
	var saveConversionXPath = "//setting[@name='saveConversion']";
	var	pasteModeXPath = "//setting[@name='pasteMode']";
	var	mesXPath = "//setting[@name='mes']";
	var	tagCSSXPath	= "//setting[@name='tagCSS']";
	var	rc, cc, rsdhref, rsdloaded, xmlDoc;
	
	// Important Objects, hold all the configuration info for the page
	var mIT = mozileInjectTarget.document;
	rc = mIT.MOZILE_RSD_CONFIG =	new	Object();
	cc = mIT.MOZILE_CURRENT_CONFIG =	new	Object();
	
	// first check in CSS
	var CSS_URL = mozileScourCSS(mozileInjectTarget);
	if(CSS_URL && CSS_URL!=="") {	
		mozile.bug.write(f,2,"Using configuration from CSS file, href="+CSS_URL);
		rsdhref = CSS_URL;
	} 
	// if that fails, check for a <link>
	else {
		mozile.bug.write(f,2,"CSS config not found. Checking for an RSD <link...>");	
		// Grab	the	head element and look for the RSD link
		// XXX Moz supports	XPath in HTML now, we could	replace	this w/	XPath
		var	links =	headnode.getElementsByTagName("link");
		for	(var i=0; i	< links.length;	i++) {
			// Find	out	if the link	element	has	a type attr	for	RSD
			var	l =	links[i];
			if (l.hasAttribute("type") && l.getAttribute("type") ==	rsdtype) {
				rsdhref	= l.getAttribute("href");
				mozile.bug.write(f,2,"RSD link found, href="+rsdhref);
			}
		}
	}
	
	rsdloaded = false;
	if ( rsdhref!=undefined && rsdhref!=='' ) {
		rsdhref=mozileAbsoluteURL(rsdhref, mozileInjectTarget.document.location); // make sure the URL is absolute
		mozile.bug.write(f,2,"Loading RSD at "+rsdhref);
		try {
  		xmlDoc = mozileInjectTarget.document.implementation.createDocument("", "", null);
			xmlDoc.async = false;
			rsdloaded = xmlDoc.load(rsdhref);
			if ( !rsdloaded ) {
					mozile.bug.write(f,1,"RSD file load has failed");
			}
		} catch (e){
			mozile.bug.write(f,1,"Exception: "+e);
		}
	}
	
	if ( rsdloaded ) {
		mozile.bug.write(f,2,"RSD file loaded");

		// If the rsd.xml has a namespace decl, the XPath breaks.
		var	evaluator =	new	XPathEvaluator();
		
		//These return an empty string if the setting name does not exist, or if it has no content.
		var	savePresetDisallow = evaluator.evaluate(presetDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveContent = evaluator.evaluate(contentXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveContentDisallow = evaluator.evaluate(contentDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethod = evaluator.evaluate(methodXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethodDisallow = evaluator.evaluate(methodDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePath = evaluator.evaluate(httpSavePathXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePathModify = evaluator.evaluate(httpSavePathModifyXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveHTMLCase = evaluator.evaluate(saveHTMLCaseXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveHTMLCompatibility = evaluator.evaluate(saveHTMLCompatibilityXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveEntities = evaluator.evaluate(saveEntitiesXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveCharset = evaluator.evaluate(saveCharsetXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveConversion = evaluator.evaluate(saveConversionXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	pasteMode	= evaluator.evaluate(pasteModeXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	mes	= evaluator.evaluate(mesXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	tagCSS = evaluator.evaluate(tagCSSXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		
		savePresetDisallow = mozileValidateConfigList( "savePreset", savePresetDisallow );
		saveContent = mozileValidateConfig( "saveContent", saveContent );
		saveContentDisallow = mozileValidateConfigList( "saveContent", saveContentDisallow );
		saveMethod = mozileValidateConfig( "saveMethod", saveMethod );
		saveMethodDisallow = mozileValidateConfigList( "saveMethod", saveMethodDisallow );
		httpSavePathModify = mozileValidateConfig( "httpSavePathModify", httpSavePathModify );
		saveHTMLCase = mozileValidateConfig( "saveHTMLCase", saveHTMLCase );

		if ( !saveHTMLCompatibility ) {
			saveHTMLCompatibility = mozileDefaultHTMLCompatibility( mIT );
		} else {
			saveHTMLCompatibility = mozileValidateConfig( "saveHTMLCompatibility", saveHTMLCompatibility );
		}

		if ( !saveCharset ) {
			saveCharset = mozileDefaultCharset( mIT );
		}
		//else Could try and check saveCharset for correctness somehow...
		
		if ( !saveEntities ) {
			saveEntities = mozileDefaultEntities( mIT );
		} else {
			saveEntities = mozileValidateConfigList( "saveEntities", saveEntities );
		}
		if ( !saveConversion ) {
			saveConversion = mozileDefaultConversion( mIT );
		} else {
			saveConversion = mozileValidateConfigList( "saveConversion", saveConversion );
		}
		

		httpSavePath = mozileAbsoluteURL(httpSavePath,rsdhref);
		if ( mes!="XHTML" ) { // make sure there are absolute URLs
			mes = mozileAbsoluteURL(mes,rsdhref);
		}
		if ( tagCSS!="XHTML" ) {
			tagCSS = mozileAbsoluteURL(tagCSS,rsdhref);
		}

		// Store the values from the RSD file
		// Store the same RSD values to the MOZILE_CURRENT_CONFIG array
		cc.saved = "true";
		cc.preset = "page";
		cc.rsdhref = rc.rsdhref = rsdhref;
		cc.savePresetDisallow = rc.savePresetDisallow = savePresetDisallow;
		cc.saveContent = rc.saveContent = saveContent;
		cc.saveContentDisallow = rc.saveContentDisallow = saveContentDisallow;
		cc.saveMethod = rc.saveMethod = saveMethod;
		cc.saveMethodDisallow = rc.saveMethodDisallow = saveMethodDisallow;
		cc.httpSavePath = rc.httpSavePath = httpSavePath;
		cc.httpSavePathModify = rc.httpSavePathModify = httpSavePathModify;
		cc.saveHTMLCase = rc.saveHTMLCase = saveHTMLCase;
		cc.saveHTMLCompatibility = rc.saveHTMLCompatibility = saveHTMLCompatibility;
		cc.saveEntities = rc.saveEntities = saveEntities;
		cc.saveCharset = rc.saveCharset = saveCharset;
		cc.saveConversion = rc.saveConversion = saveConversion;
		cc.pasteMode = rc.pasteMode = pasteMode;
		cc.mes = rc.mes = mes;
		cc.tagCSS = rc.tagCSS = tagCSS;
		mozile.bug.write(f,1,"RSD Configuration completed.");
	}
	else { // default to the built in XHTML Editing Scheme.
		mozile.bug.write(f,2,"No configuration information found, defaults will be applied");
		// Store the default values in MOZILE_CURRENT_CONFIG.
		cc.saved = "true";
		cc.preset = "user";
		cc.rsdhref = "";
		cc.savePresetDisallow = "";
		cc.saveContent = "page";
		cc.saveContentDisallow = "";
		cc.saveMethod = "display";
		cc.saveMethodDisallow = "";
		cc.httpSavePath = "";
		cc.httpSavePathModify = "true";
		cc.saveHTMLCase = "lower";
		cc.saveHTMLCompatibility = mozileDefaultHTMLCompatibility( mIT );
		cc.saveCharset = mozileDefaultCharset( mIT );
		cc.saveEntities = mozileDefaultEntities( mIT );
		cc.saveConversion = mozileDefaultConversion( mIT );
		cc.pasteMode = "text";
		cc.mes = mozileGetCurrentMES( mozileInjectTarget );
		cc.tagCSS = "XHTML";
	}
}

// The Default config values are applied if there is not a config file, or the
// settings are not in the config file.

function mozileDefaultCharset( mIT ) {
	if ( mozileIsXhtmlCT( mIT ) ) {
		return "document";
	} else {
		return "UTF-8";
	}
}

function mozileDefaultConversion( mIT ) {
	if ( mozileIsXhtmlCT( mIT ) ) {
		return mozileValidateConfigList("saveConversion", "entitybeforecharset,ignoreignorables,decimalncr");
	} else {
		return mozileValidateConfigList( "saveConversion", "none");
	}
}

function mozileDefaultEntities( mIT ) {
	if ( mozileIsXhtmlCT( mIT ) ) {
		return mozileValidateConfigList( "saveEntities", "html40latin1,html40symbols,html40special");
	} else {
		return mozileValidateConfigList( "saveEntities", "none");
	}
}

function mozileDefaultHTMLCompatibility( mIT ) {
	if ( mIT.contentType == "text/html" ) {
		return "xhtml";
	} else {
		return "xml";
	}
}

function mozileIsXhtmlCT( mIT ) {
	var cT = mIT.contentType;
	if ( (cT == "text/html") || (cT == "application/xhtml+xml") ) {
		return true;
	}
	return false;
}

//Note: All lower case.
var mozileValidConfigs = new Object();
mozileValidConfigs.savePreset = ["user","all"];
mozileValidConfigs.saveContent = ["page", "current", "current-ns"];
mozileValidConfigs.saveMethod = ["display", "local", "post", "webdav" ];
mozileValidConfigs.httpSavePathModify = ["true", "false"];
mozileValidConfigs.saveHTMLCase = ["lower", "upper"];
mozileValidConfigs.saveHTMLCompatibility = ["xml", "xhtml", "html"];
mozileValidConfigs.saveEntities = ["none", "html40latin1", "html40symbols", "html40special", "transliterate", "mathml20"];
mozileValidConfigs.saveConversion = ["none", "entitybeforecharset", "entityaftercharset", "charsetfallback",
	"ignoreignorables", "nofallback", "questionmark", "escapedunicode", "decimalncr", "hexncr"];


/**
Validates a single configuration setting.

All whitespace is removed BEFORE validating.
Not case sensitive.
Returns the default value if an invalid value is supplied, or if value is an empty string.

@return value
@type String
*/
function mozileValidateConfig( config, value ) {
	var f = ["mozileConfiguration.js","mozileValidateConfig"];

	value = value.toLowerCase();
	var re = /\s/g;
	value = value.replace( re, "" );
	if ( config in mozileValidConfigs ) {
		if ( !value ) {
			return mozileValidConfigs[config][0];
		} else {
			var j = mozileValidConfigs[config].length;
			for( var i=0; i < j; i++ ) {
				if (mozileValidConfigs[config][i] == value) {
					return value;
				}	
			}
			//If not found, return default value.
			mozile.bug.write(f,1,"Invalid RSD setting: name="+config+" value=["+value+"]");
			return mozileValidConfigs[config][0];
		}
	} else {
		return "";
	}
}

/**
Validates a comma separated list of settings.

All whitespace is removed BEFORE validating.
Not case sensitive.
Ignores duplicates.

@return a : zero length array if invalid or an empty string.
@type Array 
*/
function mozileValidateConfigList( config, csl ) {
	var f = ["mozileConfiguration.js","mozileValidateConfigList"];

	var a = new Array();
	csl = csl.toLowerCase();
	var re = /\s/g;
	csl = csl.replace( re, "" );
	if ( config in mozileValidConfigs ) {
		if ( csl ) {
			var c = mozileValidConfigs[config].length;
			a = new Array(c);
			var values = csl.split(",");
			var v = values.length;
			for ( var n=0; n < v; n++ ) {
				for( var i=0; i < c; i++ ) {
					if (mozileValidConfigs[config][i] == values[n]) {
						a[i] = mozileValidConfigs[config][i];
						break;
					}	
				}
				if ( i >= c ) { //If invalid setting
					mozile.bug.write(f,1,"Invalid RSD setting: name="+config+" value=["+values[n]+"]");
				}
			}
		}
	}
	return a;
} 

